#include <u.h>
#include <libc.h>
#include "uds.h"
/*
 * linuxemu(1) and APE compatible Unix domain socket code,
 * based on code in /sys/src/ape/lib/bsd/*.c
 *
 * The server posts pipe in /srv/UD.UD.xxxxx and reads
 * eaactly 128 bytes from it's other end.
 *
 * The client makes its own pipe and posts it into /srv/yyyyyyy,
 * and then opens the server's pipe (/srv/UD.UD.xxxxx) and 
 * writes yyyyyyy, padded to exactly 128 bytes into it.
 *
 * The padding is necessary to ensure reads can be atomic in
 * the face of several clients connecting simultaneously.
 *
 * the server now reads its 128 bytes (yyyyyyy) and opens
 * the data comms pipe /srv/yyyyyyy. 
 *
 * Finally the the server writes yyyyyyy (no padding, no 
 * trailing \0) to the data comms pipe to confirm the session 
 * is accepted; Sadly sending other text does not see to reject
 * the connection.
 */


static char *
uds(char *addr)
{
	if(strncmp(addr, "unix!", 5) == 0)
		return addr+5;
	return nil;
}

static void
srvname(char *buf, int len, char *path)
{
	char *p;
	int n;

	n = snprint(buf, len, "/srv/UD.");
	if((p = strrchr(path, '/')) == nil)
		p = path;
	else
		p++;
	snprint(buf+n, len-n, "%s", p);
}

/*
 * announce always leaks one fd - the end posted in /srv.
 * it gets cleaned up on each announce or on program exit.
 */
int
uds_announce(char *addr, char *adir)
{
	int sfd, pfd[2];
	char *name, path[128+8];

	if((name = uds(addr)) == nil)
		return announce(addr, adir);

	srvname(path, sizeof(path), name);
	remove(path);
	if((sfd = create(path, OWRITE, 0666)) < 0)
		return -1;

	if(pipe(pfd) < 0){
		close(sfd);
		return -1;
	}

	if(fprint(sfd, "%d", pfd[1]) < 0){
		close(sfd);
		close(pfd[0]);
		close(pfd[1]);
		return -1;
	}

	snprint(adir, 32, "unix!%d", pfd[0]);	/* we hope adir is big enough */
	return sfd;
}

int
uds_listen(char *adir, char *ldir)
{
	char *s, name[128], path[128+8];
	int sfd, nfd, n;

	if((s = uds(adir)) == nil)
		return -1;
	sfd = atoi(s);

	while((n = read(sfd, name, 128)) == 0)
		sleep(10);
	if(n < 0)
		return -1;
	name[n] = 0;

	srvname(path, sizeof(path), name);
	if((nfd = open(path, ORDWR)) < 0)
		return -1;

	snprint(ldir, 32, "%s", name);	/* we hope ldir is big enough */
	return nfd;
}

int
uds_accept(int nfd, char *ldir)
{
	if(write(nfd, ldir, strlen(ldir)) < 0)
		return -1;
	return dup(nfd, -1);		/* give them somthing to close */
}

int
uds_close(int fd)
{
	return close(fd);
}
